@@ -109,6 +109,12 @@ struct SPVM_compiler {
   // Method signature symbol table
   SPVM_HASH* signature_symtable;
   
+  // String pool
+  SPVM_STRING_BUFFER* string_pool;
+  
+  // String symbol table
+  SPVM_HASH* string_symtable;
+  
   char buffer1[UINT16_MAX * 2];
   char buffer2[UINT16_MAX * 2];
 };
@@ -118,7 +124,10 @@ void SPVM_COMPILER_compile(SPVM_COMPILER* compiler);
 void SPVM_COMPILER_free(SPVM_COMPILER* compiler);
 void SPVM_COMPILER_add_basic_types(SPVM_COMPILER* compiler);
 SPVM_RUNTIME* SPVM_COMPILER_new_runtime(SPVM_COMPILER* compiler);
-void SPVM_COMPILER_push_long_pool(SPVM_COMPILER* compiler, int64_t long_value);
-void SPVM_COMPILER_push_string_pool(SPVM_COMPILER* compiler, const char* string, int32_t string_length);
+void SPVM_COMPILER_error(SPVM_COMPILER* compiler, const char* message, ...);
+
+const char* SPVM_COMPILER_create_sub_signature(SPVM_COMPILER* compiler, SPVM_SUB* sub);
+const char* SPVM_COMPILER_create_field_signature(SPVM_COMPILER* compiler, SPVM_FIELD* field);
+const char* SPVM_COMPILER_create_package_var_signature(SPVM_COMPILER* compiler, SPVM_PACKAGE_VAR* package_var);
 
 #endif
