@@ -35,15 +35,18 @@ struct SPVM_package {
   SPVM_HASH* info_sub_id_symtable;
   SPVM_LIST* info_field_ids;
   SPVM_HASH* info_field_id_symtable;
-  SPVM_LIST* info_types;
-  SPVM_HASH* info_type_symtable;
+  SPVM_LIST* info_basic_type_ids;
+  SPVM_HASH* info_basic_type_id_symtable;
   SPVM_LIST* info_switch_infos;
-  SPVM_LIST* info_long_constants;
-  SPVM_HASH* info_long_constant_symtable;
-  SPVM_LIST* info_double_constants;
-  SPVM_HASH* info_double_constant_symtable;
-  SPVM_LIST* info_string_constants;
-  SPVM_HASH* info_string_constant_symtable;
+  SPVM_HASH* string_symtable;
+  SPVM_CONSTANT_POOL* constant_pool;
+  SPVM_HASH* constant_pool_32bit_value_symtable;
+  SPVM_HASH* constant_pool_64bit_value_symtable;
+  int32_t constant_pool_base;
+  int32_t no_dup_field_access_field_ids_constant_pool_id;
+  int32_t no_dup_package_var_access_package_var_ids_constant_pool_id;
+  int32_t no_dup_call_sub_sub_ids_constant_pool_id;
+  int32_t no_dup_basic_type_ids_constant_pool_id;
   const char* load_path;
   SPVM_OP* op_type;
   int32_t id;
